﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Threading;
using System.Diagnostics;
using System.Configuration;

namespace GPSd
{

    
    public class WebServer
    {

        public event ConnectionsEventHandler ConnectionsChanged;
        public delegate void ConnectionsEventHandler(object sender, int connectionscount);
    
        private int port = Convert.ToInt32(ConfigurationSettings.AppSettings["tcpport"]);
        private bool running = false;
        private Thread wsthread = null;
        List<ProcessClientRequest> connections = null;
      
        public WebServer()
        {
            connections = new List<ProcessClientRequest>();
        }

        public int Port
        {
            get { return port; }
            set { port = value; }
        }

        public void Start()
        {
            
            wsthread = new Thread(new ThreadStart(run));
            wsthread.Name = "Web Server";
            wsthread.IsBackground = true;
            wsthread.Start();


        }
        public void Stop()
        {
            if (!running) return;
            running = false;
            if (wsthread == null) return;
            wsthread.Abort();
            Debug.WriteLine("Web Server Stopped");


        }

        public void GPSMessage(string message)
        {
            Console.WriteLine(message);
            int initiallength = connections.Count;
            int intcount = 0;
            while (intcount < connections.Count)
            {
                ProcessClientRequest pc = connections[intcount];
                pc.GPSMessage(message);
                intcount++;
            }
        }

        private void run()
        {
            // Create a socket, bind it to the server's port and listen for client connections
            Socket server = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            IPEndPoint localEndPoint = new IPEndPoint(IPAddress.Any, port);
            server.Bind(localEndPoint);
            server.Listen(Int32.MaxValue);

            running = true;

            IPHostEntry mydev = Dns.GetHostEntry(Dns.GetHostName());

            for (int nindex = 0; nindex < mydev.AddressList.Length; nindex++)
            {
                string aux = mydev.AddressList[nindex].ToString();
                Console.WriteLine("Listening On " + aux + ":" + port);
            }
            while (running)
            {
                // Wait for a client to connect
                Socket clientSocket = server.Accept();

                ProcessClientRequest pc = new ProcessClientRequest(clientSocket);
                pc.ClientDisconnected += new ProcessClientRequest.DisconnectedEventHandler(pc_ClientDisconnected);
                connections.Add(pc);
                ConnectionsChanged(this, connections.Count);
            }

        }

        void pc_ClientDisconnected(object sender)
        {
            ProcessClientRequest pc = (ProcessClientRequest)sender;
            int intcount = 0;
            foreach (ProcessClientRequest p in connections)
            {
                if (p.ID == pc.ID)
                {
                    break;
                }
                intcount++;
            }
            connections.RemoveAt(intcount);
            ConnectionsChanged(this, connections.Count);
        }
    }


    internal sealed class ProcessClientRequest
    {
        public event DisconnectedEventHandler ClientDisconnected;
        public delegate void DisconnectedEventHandler(object sender);

        private string remoteip = "";
        public Guid ID = Guid.NewGuid();

        public string RemoteIP
        {
            get
            {
                return remoteip;
            }
        }

        public void GPSMessage(string message)
        {
                try
                {
                    m_clientSocket.Send(Encoding.ASCII.GetBytes(message + "\r\n"));
                }
                catch
                {
                    m_clientSocket.Close();
                    Console.WriteLine(remoteip + " Disconnected");
                    ClientDisconnected(this);
                }
        }

        private Socket m_clientSocket;

        public ProcessClientRequest(Socket clientSocket)
        {
            m_clientSocket = clientSocket;
            remoteip = m_clientSocket.RemoteEndPoint.ToString();
            remoteip = remoteip.Substring(0, remoteip.IndexOf(":"));
            Console.WriteLine(remoteip + " Just Attached");
            
        }
    }
}